@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003-2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -200,13 +200,10 @@ void Blob::print( FILE * const outfile ) const
   for( int row = top(); row <= bottom(); ++row )
     {
     for( int col = left(); col <= right(); ++col )
-      {
-      if( get_bit( row, col ) ) std::fprintf( outfile, " O" );
-      else std::fprintf( outfile, " ." );
-      }
-    std::fputs( "\n", outfile );
+      std::fputs( get_bit( row, col ) ? " O" : " .", outfile );
+    std::fputc( '\n', outfile );
     }
-  std::fputs( "\n", outfile );
+  std::fputc( '\n', outfile );
   }
 
 
