@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -20,9 +19,13 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <string>
+#include <vector>
 
 #include "common.h"
+#include "user_filter.h"
 
+int verbosity = 0;
 
 namespace {
 
@@ -42,12 +45,16 @@ struct F_entry
 
 const F_entry F_table[] =
   {
-  { "none",         Filter::none },
-  { "letters",      Filter::letters },
-  { "letters_only", Filter::letters_only },
-  { "numbers",      Filter::numbers },
-  { "numbers_only", Filter::numbers_only },
-  { 0, Filter::none }
+  { "letters",        Filter::letters },
+  { "letters_only",   Filter::letters_only },
+  { "numbers",        Filter::numbers },
+  { "numbers_only",   Filter::numbers_only },
+  { "same_height",    Filter::same_height },
+  { "text_block",     Filter::text_block },
+  { "upper_num",      Filter::upper_num },
+  { "upper_num_mark", Filter::upper_num_mark },
+  { "upper_num_only", Filter::upper_num_only },
+  { 0, Filter::user }				// not a valid name
   };
 
 struct T_entry
@@ -72,11 +79,9 @@ const T_entry T_table[] =
 } // end namespace
 
 
-int verbosity = 0;
-
 void Ocrad::internal_error( const char * const msg )
   {
-  std::fprintf( stderr, "ocrad: internal error: %s.\n", msg );
+  std::fprintf( stderr, "ocrad: internal error: %s\n", msg );
   std::exit( 3 );
   }
 
@@ -86,7 +91,7 @@ bool Ocrad::similar( const int a, const int b,
   {
   int difference = std::abs( a - b );
   if( percent_dif > 0 && difference <= abs_dif ) return true;
-  int max_abs = std::max( std::abs(a), std::abs(b) );
+  int max_abs = std::max( std::abs( a ), std::abs( b ) );
   return ( difference * 100 <= max_abs * percent_dif );
   }
 
@@ -119,36 +124,12 @@ void Charset::show_error( const char * const program_name,
   {
   if( verbosity >= 0 )
     {
-    if( arg && std::strcmp( arg, "help" ) )
+    if( arg && std::strcmp( arg, "help" ) != 0 )
       std::fprintf( stderr,"%s: bad charset '%s'\n", program_name, arg );
-    std::fputs( "Valid charset names are:", stderr );
+    std::fputs( "Valid charset names:", stderr );
     for( int i = 0; i < charsets; ++i )
       std::fprintf( stderr, "  %s", charset_name[i] );
-    std::fputs( "\n", stderr );
-    }
-  }
-
-
-bool Filter::set( const char * const name )
-  {
-  for( int i = 0; F_table[i].name != 0; ++i )
-    if( std::strcmp( name, F_table[i].name ) == 0 )
-      { type_ = F_table[i].type; return true; }
-  return false;
-  }
-
-
-void Filter::show_error( const char * const program_name,
-                         const char * const arg ) const
-  {
-  if( verbosity >= 0 )
-    {
-    if( arg && std::strcmp( arg, "help" ) )
-      std::fprintf( stderr,"%s: bad filter '%s'\n", program_name, arg );
-    std::fputs( "Valid filter names are:", stderr );
-    for( int i = 0; F_table[i].name != 0; ++i )
-      std::fprintf( stderr, "  %s", F_table[i].name );
-    std::fputs( "\n", stderr );
+    std::fputc( '\n', stderr );
     }
   }
 
@@ -167,9 +148,9 @@ void Transformation::show_error( const char * const program_name,
   {
   if( verbosity >= 0 )
     {
-    if( arg && std::strcmp( arg, "help" ) )
+    if( arg && std::strcmp( arg, "help" ) != 0 )
       std::fprintf( stderr,"%s: bad bitmap trasformation '%s'\n", program_name, arg );
-    std::fputs( "Valid transformation names are:", stderr );
+    std::fputs( "Valid transformation names:", stderr );
     for( int i = 0; T_table[i].name != 0; ++i )
       std::fprintf( stderr, "  %s", T_table[i].name );
     std::fputs( "\nRotations are made counter-clockwise.\n", stderr );
@@ -177,6 +158,51 @@ void Transformation::show_error( const char * const program_name,
   }
 
 
+Control::~Control()
+  {
+  for( unsigned f = filters.size(); f > 0; --f )
+    if( filters[f-1].user_filterp )
+      delete filters[f-1].user_filterp;
+  }
+
+
+bool Control::add_filter( const char * const program_name,
+                          const char * const name )
+  {
+  for( int i = 0; F_table[i].name != 0; ++i )
+    if( std::strcmp( name, F_table[i].name ) == 0 )
+      { filters.push_back( Filter( F_table[i].type ) ); return true; }
+  if( verbosity >= 0 )
+    {
+    if( name && std::strcmp( name, "help" ) != 0 )
+      std::fprintf( stderr,"%s: bad filter '%s'\n", program_name, name );
+    std::fputs( "Valid filter names:", stderr );
+    for( int i = 0; F_table[i].name != 0; ++i )
+      std::fprintf( stderr, "  %s", F_table[i].name );
+    std::fputc( '\n', stderr );
+    }
+  return false;
+  }
+
+
+int Control::add_user_filter( const char * const program_name,
+                              const char * const file_name )
+  {
+  const User_filter * const user_filterp = new User_filter( file_name );
+  const int retval = user_filterp->retval();
+  if( retval == 0 )
+    filters.push_back( Filter( user_filterp ) );
+  else
+    {
+    if( verbosity >= 0 )
+      std::fprintf( stderr,"%s: user filter: %s\n",
+                    program_name, user_filterp->error().c_str() );
+    delete user_filterp;
+    }
+  return retval;
+  }
+
+
 bool Control::set_format( const char * const name )
   {
   if( std::strcmp( name, "byte" ) == 0 ) { utf8 = false; return true; }
