@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -396,7 +395,7 @@ Page_image::Page_image( const Page_image & source, const int scale )
   : Rectangle( source ), maxval_( source.maxval_ ), threshold_( source.threshold_ )
   {
   if( scale < 2 || scale > source.width() || scale > source.height() )
-    Ocrad::internal_error( "bad parameter building a reduced Page_image" );
+    Ocrad::internal_error( "bad parameter building a reduced Page_image." );
 
   const int scale2 = scale * scale;
   Rectangle::height( source.height() / scale );
@@ -556,7 +555,7 @@ void Page_image::draw_track( const Track & tr )
   }
 
 
-bool Page_image::scale( int n )
+bool Page_image::change_scale( int n )
   {
   if( n <= -2 )
     { Page_image reduced( *this, -n ); *this = reduced; return true; }
