@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -163,7 +162,7 @@ int Profile::operator[]( int i )
 int Profile::area( const int l, int r )
   {
   if( limit_ < 0 ) initialize();
-  if( r < 0 ) r = samples() - 1;
+  if( r < 0 || r >= samples() ) r = samples() - 1;
   int a = 0;
   for( int i = l; i <= r; ++i ) a += data[i];
   return a;
@@ -180,14 +179,15 @@ bool Profile::increasing( int i, const int min_delta )
   }
 
 
-bool Profile::decreasing( int i )
+bool Profile::decreasing( int i, int end )
   {
   if( limit_ < 0 ) initialize();
-  const int noise = ( std::min( samples(), limit_ ) / 20 ) + 1;
-  if( i < 0 || samples() - i <= 2 * noise ||
-      data[i] - data[samples()-noise] < noise + 1 )
+  const int dnoise = ( samples() / 20 ) + 1;			// domain noise
+  const int rnoise = ( std::min( samples(), limit_ ) / 20 ) + 1; // range noise
+  if( end < 0 || end > samples() - dnoise ) end = samples() - dnoise;
+  if( i < 0 || end - i <= 2 * rnoise || data[i] - data[end-1] <= rnoise )
     return false;
-  while( ++i < samples() - noise ) if( data[i] > data[i-1] ) return false;
+  while( ++i < end ) if( data[i] > data[i-1] ) return false;
   return true;
   }
 
@@ -238,7 +238,7 @@ bool Profile::isconvex()
       }
     if( l >= r || l >= pos( 25 ) || r <= pos( 75 ) ) return isconvex_;
     if( lmin >= 0 || rmax <= 0 || data[l] < 2 || data[r] < 2 ||
-        3 * ( data[l] + data[r] ) <= std::min( limit_, samples() ) )
+        3 * ( data[l] + data[r] ) <= std::min( samples(), limit_ ) )
       return isconvex_;
     if( 3 * ( min_end - min_begin + 1 ) > 2 * samples() ) return isconvex_;
     if( 2 * l >= min_begin || 2 * r <= min_end + samples() - 1 ) return isconvex_;
@@ -371,26 +371,6 @@ bool Profile::iscpit( const int cpos )
   }
 
 
-bool Profile::islpit()
-  {
-  if( limit_ < 0 ) initialize();
-  if( samples() < 5 ) return false;
-  const int noise = samples() / 30;
-  if( data[0] < noise + 2 ) return false;
-
-  const int dmin = min();
-  int begin = 0, ref = limit_;
-  for( int i = 0; i < samples(); ++i )
-    {
-    int d = data[i];
-    if( d == dmin ) { begin = i; break; }
-    if( d < ref ) ref = d; else if( d > ref + 1 ) return false;
-    }
-  if( begin < 2 || 2 * begin >= samples() ) return false;
-  return true;
-  }
-
-
 bool Profile::istpit()
   {
   if( istpit_ < 0 )
@@ -399,7 +379,7 @@ bool Profile::istpit()
     if( limit_ < 5 || samples() < 5 || !ispit() )
       { istpit_ = false; return istpit_; }
 
-    const int noise = ( std::min( limit_, samples() ) / 30 ) + 1;
+    const int noise = ( std::min( samples(), limit_ ) / 30 ) + 1;
     int l = -1, r = 0;
     for( int i = 0; i < samples(); ++i )
       if( data[i] <= noise ) { r = i; if( l < 0 ) l = i; }
@@ -480,7 +460,8 @@ bool Profile::istip()
     int th = ( mean() < 2 && range() > 2 ) ? 2 : mean(); if( th < 2 ) ++th;
     int lth = data[0], rth = data[samples()-1];
     int begin = 0, end = samples() - 1;
-    for( int i = 1, j = std::max( 2, samples() / 10 ); i < j; ++i )
+    const int tries = std::max( 2, ( samples() + 5 ) / 10 );
+    for( int i = 1; i < tries; ++i )
       {
       if( data[i] < lth ) { lth = data[i]; begin = i; }
       if( data[samples()-1-i] < rth )
@@ -540,6 +521,46 @@ bool Profile::isctip( const int cpos )
   }
 
 
+bool Profile::isltip()
+  {
+  if( limit_ < 0 ) initialize();
+  if( samples() < 5 ) return false;
+  const int noise = ( samples() / 30 ) + 1;
+  if( data[0] <= noise ) return false;
+
+  const int dmin = min();
+  int begin = 0, ref = limit_;
+  for( int i = 0; i < samples() - noise; ++i )
+    {
+    int d = data[i];
+    if( d == dmin ) { begin = i; break; }
+    if( d < ref ) ref = d; else if( d > ref + noise ) return false;
+    }
+  if( begin <= noise || 2 * begin > samples() ) return false;
+  return true;
+  }
+
+
+bool Profile::isrtip()
+  {
+  if( limit_ < 0 ) initialize();
+  if( samples() < 5 ) return false;
+  const int noise = ( samples() / 30 ) + 1;
+  if( data[samples()-1] <= noise ) return false;
+
+  const int dmin = min();
+  int begin = 0, ref = limit_;
+  for( int i = samples() - 1; i >= noise; --i )
+    {
+    int d = data[i];
+    if( d == dmin ) { begin = samples() - 1 - i; break; }
+    if( d < ref ) ref = d; else if( d > ref + noise ) return false;
+    }
+  if( begin <= noise || 2 * begin > samples() ) return false;
+  return true;
+  }
+
+
 int Profile::imaximum()
   {
   if( limit_ < 0 ) initialize();
