@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -50,19 +49,20 @@ public:
 
   int  area( const int l = 0, int r = -1 );
   bool increasing( int i = 1, const int min_delta = 2 );
-  bool decreasing( int i = 1 );
+  bool decreasing( int i = 1, int end = -1 );
   bool isconcave();
   bool isconvex();
   bool isflat();
   bool isflats();
   bool ispit();
   bool iscpit( const int cpos = 50 );
-  bool islpit();
   bool istpit();
   bool isupit();
   bool isvpit();
   bool istip();
   bool isctip( const int cpos = 50 );
+  bool isltip();
+  bool isrtip();
   int  imaximum();
   int  iminimum( const int m = 0, int th = -1 );
   int  minima( int th = -1 );
