@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -27,6 +26,8 @@
 #include "rational.h"
 #include "rectangle.h"
 #include "track.h"
+#include "ucs.h"
+#include "user_filter.h"
 #include "bitmap.h"
 #include "blob.h"
 #include "character.h"
@@ -89,6 +90,60 @@ void join_characters( std::vector< Textline * > & tlpv )
 } // end namespace
 
 
+void Textblock::apply_filters( const Control & control )
+  {
+  if( textlines() <= 0 ) return;
+  for( unsigned f = 0; f < control.filters.size(); ++f )
+    {
+    if( control.filters[f].user_filterp )
+      {
+      for( int i = 0; i < textlines(); ++i )
+        tlpv[i]->apply_user_filter( *control.filters[f].user_filterp );
+      continue;
+      }
+    const Filter::Type filter = control.filters[f].type;
+    if( filter != Filter::text_block )
+      {
+      for( int i = 0; i < textlines(); ++i )
+        tlpv[i]->apply_filter( filter );
+      continue;
+      }
+    int l = right(), t = bottom(), r = left(), b = top();
+    for( int i = 0; i < textlines(); ++i )
+      {
+      const Textline & line = *tlpv[i];
+      int first = line.characters(), last = -1;
+      for( int j = line.big_initials(); j < line.characters(); ++j )
+        if( line.is_key_character( j ) )
+          { l = std::min( l, line.character(j).left() ); first = j; break; }
+      for( int j = line.characters() - 1; j >= first; --j )
+        if( line.is_key_character( j ) )
+          { r = std::max( r, line.character(j).right() ); last = j; break; }
+      if( i == 0 )
+        { for( int j = first; j <= last; ++j )
+            if( line.is_key_character( j ) )
+              t = std::min( t, line.character(j).top() ); }
+      else if( i == textlines() - 1 )
+        { for( int j = first; j <= last; ++j )
+            if( line.is_key_character( j ) )
+              b = std::max( b, line.character(j).bottom() ); }
+      }
+    if( r < l || b < t ) continue;		// can't apply filter; no text
+    Rectangle re( l, t, r, b );
+    for( int i = 0; i < textlines(); ++i )
+      {
+      Textline & line = *tlpv[i];
+      bool modified = false;
+      for( int j = line.characters() - 1; j >= 0; --j )
+        if( line.character(j).height() >= 2 * line.height() ||
+            !re.includes( line.character(j).vcenter(), line.character(j).hcenter() ) )
+          { line.delete_character( j ); modified = true; }
+      if( modified ) line.remove_leadind_trailing_duplicate_spaces();
+      }
+    }
+  }
+
+
 Textblock::Textblock( const Rectangle & page, const Rectangle & block,
                       std::vector< Blob * > & blobp_vector )
   : Rectangle( block )
@@ -158,6 +213,7 @@ Textblock::Textblock( const Rectangle & page, const Rectangle & block,
   blobp_vector.clear();
 
   // Assign normal blobs to characters and create lines.
+  {
   int min_line = 0;			// first line of current cut
   tlpv.push_back( new Textline );
   int current_line = min_line = textlines() - 1;
@@ -275,6 +331,7 @@ Textblock::Textblock( const Rectangle & page, const Rectangle & block,
       if( lp->character( i ).top() <= t && 2 * ++c >= lp->characters() )
         { delete_line( tlpv, 0 ); break; }
     }
+  }
 
   // Second pass. Join lines of i-dots and tildes.
   for( int current_line = 0; current_line < textlines() - 1; )
@@ -359,20 +416,18 @@ Textblock::~Textblock()
   }
 
 
-void Textblock::recognize( const Charset & charset, const Filter & filter )
+void Textblock::recognize( const Control & control )
   {
   // Recognize characters.
   for( int i = 0; i < textlines(); ++i )
     {
     // First pass. Recognize the easy characters.
-    tlpv[i]->recognize1( charset );
+    tlpv[i]->recognize1( control.charset );
     // Second pass. Use context to clear up ambiguities.
-    tlpv[i]->recognize2( charset );
+    tlpv[i]->recognize2( control.charset );
     }
 
-  if( filter.type() != Filter::none )
-    for( int i = 0; i < textlines(); ++i )
-      tlpv[i]->apply_filter( filter );
+  apply_filters( control );
 
   // Remove unrecognized lines.
   for( int i = textlines() - 1; i >= 0; --i )
@@ -417,7 +472,7 @@ void Textblock::recognize( const Charset & charset, const Filter & filter )
 const Textline & Textblock::textline( const int i ) const
   {
   if( i < 0 || i >= textlines() )
-    Ocrad::internal_error( "line, index out of bounds" );
+    Ocrad::internal_error( "line, index out of bounds." );
   return *tlpv[i];
   }
 
@@ -435,7 +490,7 @@ void Textblock::print( const Control & control ) const
   {
   for( int i = 0; i < textlines(); ++i )
     tlpv[i]->print( control );
-  std::fputs( "\n", control.outfile );
+  std::fputc( '\n', control.outfile );
   }
 
 
@@ -449,7 +504,7 @@ void Textblock::dprint( const Control & control, bool graph, bool recursive ) co
                   tlpv[i]->characters(), i + 1 );
     tlpv[i]->dprint( control, graph, recursive );
     }
-  std::fputs( "\n", control.outfile );
+  std::fputc( '\n', control.outfile );
   }
 
 
