@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -25,10 +24,12 @@
 #include <stdint.h>
 
 #include "common.h"
+#include "histogram.h"
 #include "rational.h"
 #include "rectangle.h"
 #include "track.h"
 #include "ucs.h"
+#include "user_filter.h"
 #include "bitmap.h"
 #include "blob.h"
 #include "character.h"
@@ -38,7 +39,7 @@
 
 namespace {
 
-// Return the character position >= first preceding a big gap or eol.
+// Returns the character position >= first preceding a big gap or eol.
 //
 int find_big_gap( const Textline & line, const int first,
                   const int space_width_limit )
@@ -108,7 +109,7 @@ void Textline::verify_big_initials()
 Character & Textline::character( const int i ) const
   {
   if( i < 0 || i >= characters() )
-    Ocrad::internal_error( "character, index out of bounds" );
+    Ocrad::internal_error( "character, index out of bounds." );
   return *cpv[i];
   }
 
@@ -127,10 +128,19 @@ Rectangle Textline::charbox( const Character & c ) const
   }
 
 
+bool Textline::is_key_character( const int i ) const
+  {
+  if( i < big_initials_ || i >= characters() )
+    Ocrad::internal_error( "is_key_character, index out of bounds." );
+  return ( cpv[i]->isalnum() && cpv[i]->guess( 0 ).code != 'J' &&
+           cpv[i]->height() < 2 * height() && 2 * cpv[i]->height() > height() );
+  }
+
+
 void Textline::delete_character( const int i )
   {
   if( i < 0 || i >= characters() )
-    Ocrad::internal_error( "delete_character, index out of bounds" );
+    Ocrad::internal_error( "delete_character, index out of bounds." );
   if( i < big_initials_ ) --big_initials_;
   delete cpv[i]; cpv.erase( cpv.begin() + i );
   }
@@ -151,9 +161,9 @@ int Textline::shift_characterp( Character * const p, const bool big )
 bool Textline::insert_space( const int i, const bool tab )
   {
   if( i <= 0 || i >= characters() )
-    Ocrad::internal_error( "insert_space, index out of bounds" );
+    Ocrad::internal_error( "insert_space, index out of bounds." );
   if( !height() )
-    Ocrad::internal_error( "insert_space, track not set yet" );
+    Ocrad::internal_error( "insert_space, track not set yet." );
   Character & c1 = *cpv[i-1];
   Character & c2 = *cpv[i];
   int l = c1.right() + 1;
@@ -284,7 +294,7 @@ int Textline::mean_vcenter() const
 void Textline::print( const Control & control ) const
   {
   for( int i = 0; i < characters(); ++i ) character( i ).print( control );
-  std::fputs( "\n", control.outfile );
+  std::fputc( '\n', control.outfile );
   }
 
 
@@ -292,8 +302,14 @@ void Textline::dprint( const Control & control, const bool graph,
                        const bool recursive ) const
   {
   if( graph || recursive )
-    std::fprintf( control.outfile, "mean height = %d, track segments = %d\n",
-                  mean_height(), segments() );
+    {
+    Histogram hist;
+    for( int i = 0; i < characters(); ++i )
+      if( !character(i).maybe(' ') )
+        hist.add_sample( character(i).height() );
+    std::fprintf( control.outfile, "mean height = %d, median height = %d, track segments = %d, big initials = %d\n",
+                  mean_height(), hist.median(), segments(), big_initials_ );
+    }
 
   for( int i = 0; i < characters(); ++i )
     {
@@ -301,7 +317,7 @@ void Textline::dprint( const Control & control, const bool graph,
     if( i < big_initials_ ) c.dprint( control, c, graph, recursive );
     else c.dprint( control, charbox( c ), graph, recursive );
     }
-  std::fputs( "\n", control.outfile );
+  std::fputc( '\n', control.outfile );
   }
 
 
@@ -339,19 +355,67 @@ void Textline::recognize1( const Charset & charset ) const
   }
 
 
-void Textline::apply_filter( const Filter & filter )
+void Textline::apply_filter( const Filter::Type filter )
   {
-  bool flag = false;
+  bool modified = false;
+  if( filter == Filter::same_height )
+    {
+    Histogram hist;
+    for( int i = 0; i < characters(); ++i )
+      if( !character(i).maybe(' ') )
+        hist.add_sample( character(i).height() );
+    const int median_height = hist.median();
+    for( int i = characters() - 1; i >= 0; --i )
+      if( !character(i).maybe(' ') &&
+          !Ocrad::similar( character(i).height(), median_height, 10, 2 ) )
+        { delete_character( i ); modified = true; }
+    }
+  else
+    {
+    for( int i = characters() - 1; i >= 0; --i )
+      {
+      Character & c = character( i );
+      if( !c.guesses() ) continue;
+      c.apply_filter( filter );
+      if( !c.guesses() && filter != Filter::upper_num_mark )
+        { delete_character( i ); modified = true; }
+      }
+    if( filter == Filter::upper_num_mark )
+      join_broken_unrecognized_characters();
+    }
+  if( modified ) remove_leadind_trailing_duplicate_spaces();
+  }
+
+
+void Textline::apply_user_filter( const User_filter & user_filter )
+  {
+  bool modified = false;
   for( int i = characters() - 1; i >= 0; --i )
     {
     Character & c = character( i );
     if( !c.guesses() ) continue;
-    c.apply_filter( filter );
-    if( !c.guesses() ) { delete_character( i ); flag = true; }
+    c.apply_user_filter( user_filter );
+    if( !c.guesses() && user_filter.discard() )
+      { delete_character( i ); modified = true; }
     }
-  if( flag )			// remove leadind/trailing/duplicate spaces
-    for( int i = characters() - 1; i >= 0; --i )
-      if( character(i).maybe(' ') &&
-          ( i == 0 || i == characters() - 1 || character(i-1).maybe(' ') ) )
-        delete_character( i );
+  if( user_filter.mark() ) join_broken_unrecognized_characters();
+  if( modified ) remove_leadind_trailing_duplicate_spaces();
+  }
+
+
+void Textline::join_broken_unrecognized_characters()
+  {
+  for( int i = characters() - 1; i > 0; --i )
+    if( !character(i).guesses() &&
+        character(i).h_overlaps( character( i - 1 ) ) )
+      delete_character( i );
+  }
+
+
+void Textline::remove_leadind_trailing_duplicate_spaces()
+  {
+  for( int i = characters() - 1; i >= 0; --i )
+    if( character(i).maybe(' ') &&
+        ( i == 0 || i == characters() - 1 || character(i-1).maybe(' ') ) )
+      delete_character( i );
   }
