@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003-2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,6 +41,8 @@ public:
   int width() const
     { return cpv.empty() ? 0 : cpv.back()->right() - cpv.front()->left(); }
 
+  bool is_key_character( const int i ) const;	// isalnum of normal size
+
   void delete_character( const int i );
   int  shift_characterp( Character * const p, const bool big = false );
   bool insert_space( const int i, const bool tab = false );
@@ -62,4 +64,7 @@ public:
   void recognize1( const Charset & charset ) const;
   void recognize2( const Charset & charset );
   void apply_filter( const Filter::Type filter );
+  void apply_user_filter( const User_filter & user_filter );
+  void join_broken_unrecognized_characters();
+  void remove_leadind_trailing_duplicate_spaces();
   };
