@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003-2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -68,6 +68,6 @@ public:
   int top( const int col ) const;
   int vcenter( const int col ) const;
 
-  bool includes( const Rectangle & r ) const;
-  bool includes( const int row, const int col ) const;
+//  bool includes( const Rectangle & r ) const;
+//  bool includes( const int row, const int col ) const;
   };
